@@ -4,7 +4,8 @@ use strict;
 use warnings;
 
 use Carp 'confess';
-use Scalar::Util 'blessed';
+use Devel::Caller 'caller_cv';
+use Scalar::Util 'blessed', 'refaddr';
 
 use Moose::Util::TypeConstraints qw( find_type_constraint class_type role_type );
 use Params::Validate             ();
@@ -18,7 +19,7 @@ use Sub::Exporter -setup => {
     },
 };
 
-our $VERSION   = '0.11';
+our $VERSION   = '0.13';
 our $AUTHORITY = 'cpan:STEVAN';
 
 my %CACHED_SPECS;
@@ -51,7 +52,7 @@ sub validated_hash {
     my %args = @$args;
 
     $args{$_} = $spec{$_}{constraint}->coerce( $args{$_} )
-        for grep { $spec{$_}{coerce} } keys %spec;
+        for grep { $spec{$_}{coerce} && exists $args{$_} } keys %spec;
 
     %args = Params::Validate::validate_with(
         params => \%args,
@@ -145,7 +146,7 @@ sub pos_validated_list {
     my @args = @{$args};
 
     $args[$_] = $pv_spec[$_]{constraint}->coerce( $args[$_] )
-        for grep { $pv_spec[$_]{coerce} } 0 .. $#pv_spec;
+        for grep { $pv_spec[$_] && $pv_spec[$_]{coerce} } 0 .. $#args;
 
     @args = Params::Validate::validate_with(
         params => \@args,
@@ -163,7 +164,7 @@ sub _cache_key {
         return delete $spec->{MX_PARAMS_VALIDATE_CACHE_KEY};
     }
     else {
-        return _caller_name(1);
+        return refaddr( caller_cv(2) );
     }
 }
 
